﻿using System;
using System.Linq;
using GalaSoft.MvvmLight;
using System.ComponentModel.Composition;
using GalaSoft.MvvmLight.Messaging;
using GalaSoft.MvvmLight.Command;
using MVVM.Framework;
using Northwind.Common;
using System.Diagnostics;

using System.Windows;
using Northwind.Common.NwSvc;

namespace Northwind.Modules.Orders
{
    [Export(typeof(IOrderViewModel))]
    public class OrderViewModel : ViewModelBase, IOrderViewModel, ICleanup
    {
        #region Private fields

        private string _customerFilterText;

        private SmartObservableCollection<CustomerModel> _customersOrigCopy;

        private SmartObservableCollection<CustomerModel> _customers;
        private INWClientService _nwService;
        
        private int _orderCount;
        private SmartObservableCollection<OrderModel> _orders;

        private RelayCommand<CustomerModel> _customerChkBoxClickedCommand;

        private RelayCommand _orderClickedCommand;
        private RelayCommand _refreshOrdersCommand;
        private RelayCommand _mapOrdersCommand;


        #endregion

        #region ctor

        //[ImportingConstructor]
        //public OrderViewModel(INorthwindClientService nwService)
        public OrderViewModel()
        {
            //Orders.ElementAt(0).ShipAddress
            _nwService = new Northwind.Common.NWClientService();
            Orders = new SmartObservableCollection<OrderModel>();
            Customers = new SmartObservableCollection<CustomerModel>();
            _customersOrigCopy = new SmartObservableCollection<CustomerModel>();

            PopulateCustomers();
        }

        #endregion

        #region Public Properties

        public string CustomerFilterText
        {
            get { return _customerFilterText; }
            set
            {
                if (_customerFilterText != value)
                {
                    if (string.IsNullOrEmpty(value))
                    {
                        Customers.Repopulate(_customersOrigCopy);
                    }
                    else
                    {
                        Customers.Repopulate(_customersOrigCopy.Where(w => w.CustomerId.DataValue.ToLower().StartsWith(value.ToLower())));

                        var _baseCustomerAll = Customers.FirstOrDefault(w => w.CustomerId.DataValue.ToLower() == "all");

                        if (_baseCustomerAll == null)
                        {
                            CustomerModel _baseall = new CustomerModel();
                            _baseall.CustomerId.DataValue = "All";

                            Customers.Insert(0, _baseall);
                        }
                    }

                    _customerFilterText = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => CustomerFilterText));
                }
            }
        }

        public RelayCommand<CustomerModel> CustomerChkBoxClickedCommand
        {
            get
            {
                return _customerChkBoxClickedCommand ?? (_customerChkBoxClickedCommand = new RelayCommand<CustomerModel>((a) =>
                    {
                        // If a user selects the all button.  Set the rest of the values accordingly
                        if (a.CustomerId.DataValue == "All")
                        {
                            foreach (var t in Customers)
                            {
                                t.IsSelected = a.IsSelected;
                            }
                        }
                    }));
            }
        }
        
        public SmartObservableCollection<CustomerModel> Customers
        {
            get { return _customers; }
            set
            {
                if (_customers != value)
                {
                    _customers = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => Customers));
                }
            }
        }

        public RelayCommand MapOrdersCommand
        {
            get
            {
                return _mapOrdersCommand ?? (_mapOrdersCommand = new RelayCommand(() =>
                {
                    SmartObservableCollection<OrderModel> _copyOfOrderModels = new SmartObservableCollection<OrderModel>();

                    _copyOfOrderModels.Repopulate(Orders);

                    OrderController.ShowMapOrderView(_copyOfOrderModels);
                }
                    ));
            }
        }

        public IOrderController OrderController
        {
            get;
            set;
        }

        public RelayCommand OrderClickedCommand
        {
            get { return _orderClickedCommand ?? (_orderClickedCommand = new RelayCommand(() => OrderClicked())); }
        }

        
        public int OrderCount
        {
            get { return _orderCount; }
            set
            {
                if(_orderCount != value )
                {
                _orderCount = value;
                RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => OrderCount));
                }
            }
        }

        public SmartObservableCollection<OrderModel> Orders
        {
            get { return _orders; }
            set
            {
                if (_orders != value)
                {
                    _orders = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => Orders));
                }
            }
        }

        public RelayCommand RefreshOrdersCommand
        {
            get { return _refreshOrdersCommand ?? (_refreshOrdersCommand = new RelayCommand(() => RefreshOrders())); }
        }


        #endregion

        #region Private Methods

        private void OrderClicked()
        {
            Debug.WriteLine("Order Clicked");
        }
                
        void NWService_GetCustomersComplete(object sender, ServiceResultArgs<CustomerModel> e)
        {
            try
            {
                // Unscribe to the event so code doesn't run from other modules making this call
                _nwService.GetCustomersComplete -= new EventHandler<ServiceResultArgs<CustomerModel>>(NWService_GetCustomersComplete);

                if (!e.HasError)
                {
                    if (e.Results != null && e.Results.Count > 0)
                    {
                        Customers.Repopulate(e.Results);
                        CustomerModel _cm = new CustomerModel();
                        _cm.CustomerId.DataValue = "All";

                        Customers.Insert(0, _cm);

                        _customersOrigCopy.Repopulate(Customers);
                    }

                }
                else
                    throw e.Error;
            }
            catch (Exception ex)
            {
                GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(ex, this.GetType().ToString()));
            }
            finally
            {
                e.Results.Clear();
                e = null;
                GlobalAppCommands.SetAppIsBusyStatus.Send(null);
            }
        }
            
        void NWService_GetOrdersComplete(object sender, ServiceResultArgs<OrderModel> e)
        {
            try
            {
                // Unscribe to the event so code doesn't run from other modules making this call
                _nwService.GetOrdersComplete -= new EventHandler<ServiceResultArgs<OrderModel>>(NWService_GetOrdersComplete);

                if (!e.HasError)
                {
                    if (e.Results != null && e.Results.Count > 0)
                    {
                        //Orders[0].
                        Orders.Repopulate(e.Results);
                        OrderCount = Orders.Count;
                    }
                        
                }
            }
            catch (Exception ex)
            {
                GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(ex, this.GetType().ToString()));
            }
            finally
            {
                e.Results.Clear();
                e = null;
                GlobalAppCommands.SetAppIsBusyStatus.Send(null);
            }
        }

        private void PopulateCustomers()
        {
            _nwService.GetCustomersComplete += new EventHandler<ServiceResultArgs<CustomerModel>>(NWService_GetCustomersComplete);
            GlobalAppCommands.SetAppIsBusyStatus.Send(new SetAppIsBusyStatusEvent("Retrieving customers..", true));
            _nwService.GetCustomersAsync(new GetCustomersRequest());
        }

        private void RefreshOrders()
        {
            var _customerIds = Customers.Where(c => c.IsSelected).Select(c => c.CustomerId.DataValue).ToList();

            if (_customerIds == null || _customerIds.Count < 1)
            {
                MessageBox.Show("Please select at least one customer");
            }
            else
            {

                _nwService.GetOrdersComplete += new EventHandler<ServiceResultArgs<OrderModel>>(NWService_GetOrdersComplete);
                GlobalAppCommands.SetAppIsBusyStatus.Send(new SetAppIsBusyStatusEvent("Retrieving orders..", true));

                GetOrdersRequest _request = new GetOrdersRequest();
                _request.CustomerIds = _customerIds.ToArray();

                _nwService.GetOrdersAsync(_request);
            }
        }

        #endregion
        
        #region Public Methods

        public void Initialize()
        {
            
        }

        
        public override void Cleanup()
        {
            base.Cleanup();
            Messenger.Default.Unregister(this);

        }

        #endregion
    }
}
